clear j
close all;
c = ureal('c',0.5,'range',[0 1]);
G11=tf([6 6],[1 5 6]);
G12=tf([c],[1 20]);
G21=G12;
G22=tf([0.1 1],[1 2 1]);

G=[G11 G12;G21 G22];
G12nom=tf([c.Nominal],[1 20]);
Gnom=[G11 G12nom;G12nom G22];

wc=5;
K1=tf([wc],[1 0])*inv(G11);
K2=tf([wc],[1 0])*inv(G22);
K=[K1 0;0 K2];

Lnom=Gnom*K;
L=G*K;
Unit=eye(2);
Tnom=feedback(Lnom,Unit);
T=feedback(L,Unit);
Snom=Unit-Tnom;

t=(0:1000)*0.01/wc;
u=cos(0*t);

w=logspace(-5,5,101);
wB=3; % desired closed-loop bandwidth
A=1e-3; % desired disturbance attenuation inside bandwidth
M=1.7; % desired bound on hinfnorm(S) & hinfnorm(T)
n=1;    % the slope of the weight function
WP=zpk([-wB*M^(1/n)*ones(n,1)],[-wB*A^(1/n)*ones(n,1)],1/M);

figure(99);
hinf_S = hinfplot(Snom,w);
hinf_wp=hinfplot(WP,w);
loglog(w,hinf_S,'b');
hold on;
loglog(w,1./hinf_wp,'r');
hold off;
xlabel('\omega')
title('\sigma_{max}(S) (blue) vs 1/|W_p| (red)')

figure(1)
lsim(Tnom,[1*u' 1*u'],t);
title('Nominal Close Loop Plant Step Response')

cr=0:0.1:1;
Nr=length(cr);
for i=1:Nr
    G12=tf([cr(i)],[1 20]);
    G=[G11 G12;G12 G22];
    L=G*K;
    Unit=eye(2);
    T=feedback(L,Unit);
    figure(2)
    lsim(T,[1*u' 1*u'],t);
    hold on;
end;
title('Perturbed Close Loop Plant Step Responses')

zz=-0.15;
pp=-50;
Wo=zpk([zz],[pp],0.03*pp/zz);
hinf_T = hinfplot(Tnom,w);
hinf_wo=hinfplot(Wo,w);
figure(3)
loglog(w,hinf_T,'b');
hold on;
loglog(w,1./hinf_wo,'r');
hold off;
xlabel('\omega')
title('\sigma_{max}(T) (blue) vs 1/|W_o| (red)')

blk=[2 0];
Ws=[0 tf(0.5,[1 20]);tf(0.5,[1 20]) 0];
M = Ws*K*Snom;
Mf=frd(M,w);
[mubnds,muinfo]=mussv(Mf,blk,'c');
figure(4);
bodemag(mubnds(:,1),'r',mubnds(:,2),'b');
title('\mu(M(j\omega))');
xlabel('\omega');

figure(5)
Nw=length(w);
data_a=zeros(Nw,2);
for i=1:Nw
    MM=evalfr(M,j*w(i));
    data_a(i,:)=eig(MM);
end;
plot(data_a(:,1),'r');
hold on;
plot(data_a(:,2),'b');
hold off;
title('The eigen values of M(j\omega)');
xlabel('\omega');


